<?php

namespace TUTOR_PRO;

if (!defined('ABSPATH'))
    exit;

class Enrollment_Expiry {
    function __construct() {
        add_filter( 'tutor_course_settings_tabs', array($this, 'settings_attr') );
        add_action( 'single_course_template_before_load', array($this, 'cancel_expired_enrolment'), 10, 1 );
        add_action( 'tutor_before_enrolment_check', array($this, 'cancel_expired_enrolment'), 10, 2 );

        add_action( 'tutor_course/single/add-to-cart/after', array($this, 'show_expires_info_not_enrolled') );
        add_action( 'tutor_enrolled_box_after', array($this, 'show_expires_info') );
    }

	public function settings_attr($args) {
		$args['general']['fields']['enrollment_expiry'] = array(
            'type' => 'number',
            'label' => __('Enrollment Period', 'tutor-pro'),
            'default' => '0',
            'desc' => __('Enrollment will expire after this number of days. Set 0 for no expiration.', 'tutor-pro')
		);
		return $args;
    }

    public function cancel_expired_enrolment($course_id, $user_id=null) {

        if(!$user_id && !is_user_logged_in(  )) {
            return;
        }

        global $wpdb;

        $expiry = get_tutor_course_settings($course_id, 'enrollment_expiry');
        if(!is_numeric($expiry) || $expiry<1) {
            return;
        }

        $expired_date = tutor_time() - ((60*60*24)*$expiry);
        $current_id = $user_id ? $user_id : get_current_user_id(  );

        $ids = $wpdb->get_col(
            "SELECT ID FROM {$wpdb->posts} 
            WHERE post_author={$current_id} 
                AND post_parent={$course_id} 
                AND post_type='tutor_enrolled' 
                AND post_status='completed' 
                AND UNIX_TIMESTAMP(post_date)<{$expired_date}");

        if( is_array( $ids ) && count( $ids ) ) {

            $wpdb->query("UPDATE {$wpdb->posts} SET post_status='cancel' WHERE ID IN ( " . implode(',', $ids) . " )");

            foreach($ids as $id) {
                do_action( 'tutor_enrollment/after/expired', $id );
            }
        }
    }

    public function show_expires_info_not_enrolled() {

        $course_id = get_the_ID();
        $expiry = get_tutor_course_settings($course_id, 'enrollment_expiry');

        if (is_numeric( $expiry ) || $expiry>=1) {
            $day = $expiry>1 ? __('days', 'tutor-pro') : __('day', 'tutor-pro');

            echo '<p class="enrolment-expire-info"><i class="tutor-icon-calendar"></i> ', __('Enrolment validity', 'tutor-pro'), ': <b>', $expiry, ' ', $day, '</b></p>';
        }
    }

    public function show_expires_info($ret=false) {
        
        $course_id = get_the_ID();
        $expiry = get_tutor_course_settings($course_id, 'enrollment_expiry');
        
        if (!is_numeric( $expiry ) || $expiry<1 || !is_user_logged_in()) {
            return;
        }

        $enrolment = tutils()->is_enrolled($course_id, get_current_user_id());
        
        if ( $enrolment ) {

            $date = date_create( $enrolment->post_date );
            date_add( $date, date_interval_create_from_date_string($expiry . ' days') );
    
            $validity = date_format($date, get_option('date_format'));
            $text = __('Enrollment valid until', 'tutor-pro');

            echo '<p class="enrolment-expire-info"><i class="tutor-icon-calendar"></i> '.$text . ' <b>' . $validity .'</b></p>';
        }
    }
}